﻿--
-- OpenRaid (c) OpenRaid.eu/OpenRaid.eu.
-- By: Kimpossibru-Khadgar EU
--
-- Version 1.1 - 14 April 2012
--
-- Progress bar vars
local or_progressValue = 100;
local or_progressCounter = 0;
local or_progressing = false;

-- Invite list vars
local OR_inviteList = {};
local OR_inviteIndex = 1;

function or_Init()
	if (or_invmsg == nil) then
		or_invmsg = "Hi! I'm adding you for the scheduled OpenRaid raid!";
	end
end

-- Make OpenRaid slash commands (/openraid) to open the main window.
SLASH_OPENRAID1, SLASH_OPENRAID2 = '/oraid', '/openraid';
function SlashCmdList.OPENRAID(msg, editbox)
	
	if (msg == 'invite' or msg == 'inv') then
		print('|cffB40404[OpenRaid]: |cffB18904Opened invites window');
		
		if (OR_inviteIndex == 1 or OR_inviteIndex == nil) then
			OpenRaidImportUserEdit:SetText("");
			OpenRaidInvForm:Show();
		else
			OR_ScrollIndex = 0;
			OR_startInvites();
		end
	elseif (strsub(msg, 0, 3) == 'msg') then
		or_invmsg = strsub(msg, 4);
		print('|cffB40404[OpenRaid]: |cffB18904');
		print('|cffB40404[OpenRaid]: |cffB18904Your personal message is now set to:');
		print('|cffB40404[OpenRaid]: |cffB18904'..or_invmsg);
	elseif (msg == 'import') then
		print('|cffB40404[OpenRaid]: |cffB18904Opened import window');
		OpenRaidForm:Show();
	else
		print('|cffB40404[OpenRaid]: |cffB18904Your personal message is:');
		print('|cffB40404[OpenRaid]: |cffB18904'..or_invmsg);
		print('|cffB40404[OpenRaid]: |cffB18904');
		print('|cffB40404[OpenRaid]: |cffB18904Available commands:');
		print('|cffB40404[OpenRaid]: |cffB18904 /oraid inv: import your own raids');
		print('|cffB40404[OpenRaid]: |cffB18904 /oraid import: import your own raids');
		print('|cffB40404[OpenRaid]: |cffB18904 /oraid msg [msg]: change your invite message to [msg]');
		
	end
end

-- Called when a checkbox is clicked to reinvite everyone. yeye can just use the below feature but, ohwell!
function OR_bnetInvitesOnly()

	-- Invite all the users
	local orString = {strsplit(";", OpenRaidImportUserEdit:GetText())};
	
	for n=1,getn(orString) do
	
		-- Last string is empty.
		if (orString[n] == nil or orString[n] == '') then
			break;
		end
		
		-- user[1] is the email address, user[2] is the user's name.
		local user = {strsplit(",", orString[n])};
				
		-- Don't Bnet invite people on the same realm
		if (user[1] ~= 'SAME_REALM' and user[1] ~= nil) then
			-- As email addresses are hidden, the only way to make sure everyone is added is to try to add them again...
			BNSendFriendInvite(user[1], or_invmsg);
		end
		
		-- I would like to set friend notes here, but the user would have to accept first... This is kind of kIMPOSSIBRU!
		-- BNSetFriendNote(ID, noteText)
	end
end

-- Called when an invitestring is passed
function OR_importUsers()

	-- Clear the to-be-invited list.
	OR_inviteList = {};
	OR_inviteIndex = 1;
	
	-- Invite all the users
	local orString = {strsplit(";", OpenRaidImportUserEdit:GetText())};
	
	for n=1,getn(orString) do
	
		-- Last string is empty.
		if (orString[n] == nil or orString[n] == '') then
			break;
		end
		
		-- user[1] is the email address, user[2] is the user's name.
		local user = {strsplit(",", orString[n])};
		
		-- Add to the to-be-invited list.
		if (user[2] ~= nil) then
			OR_inviteList[OR_inviteIndex] = user[2];
			OR_inviteIndex = OR_inviteIndex + 1;
		end
		
		-- Don't Bnet invite people on the same realm
		if (user[1] ~= 'SAME_REALM' and user[1] ~= nil) then
			-- As email addresses are hidden, the only way to make sure everyone is added is to try to add them again...
			BNSendFriendInvite(user[1], or_invmsg);
		end
		
		-- I would like to set friend notes here, but the user would have to accept first... This is kind of kIMPOSSIBRU!
		-- BNSetFriendNote(ID, noteText)
	end
	
	OR_startInvites();
end

function OR_startInvites()
	
	--BNSetCustomMessage("Starting OpenRaid invites!");
	OpenRaidInvForm:Hide();
	OpenRaidInvList:Show();
	OR_updateChecks();
	
end

local OR_ScrollIndex = 0;
function OR_scrollDown()
	if (OR_ScrollIndex < getn(OR_inviteList) - 1) then
		OR_ScrollIndex = OR_ScrollIndex + 1;
		OR_updateChecks();
	end
end

function OR_scrollUp()
	if (OR_ScrollIndex > 0) then
		OR_ScrollIndex = OR_ScrollIndex - 1;
		OR_updateChecks();
	end
end

function OR_resetInvs()
	OR_inviteIndex = 1;
	OR_inviteList = {};
	wipe(OR_inviteList);
	OpenRaidImportUserEdit:SetText("");
end

-- Update the checkbox texts
function OR_updateChecks()
	
	for i=1,5 do
		_G["OR_Chk"..i]:SetChecked(0);
		_G["OR_Lbl"..i]:SetText('');
		_G["OR_Lbl"..i]:Hide();
	end
	
	local chkbox = 1;
	
	for index,value in pairs(OR_inviteList) do 
		
		if index > OR_ScrollIndex then
			-- There are only 5 checkboxes
			if (chkbox > 5) then
				break;
			end
			
			_G["OR_Chk"..chkbox]:SetChecked(0);
			_G["OR_Lbl"..chkbox]:SetText(value);
			_G["OR_Lbl"..chkbox]:Show();
			
			chkbox = chkbox + 1;
		end
	end
end


-- Clicked on a checkbox in the invites list
function OR_clickedChk(index)
	
	tremove(OR_inviteList, index+OR_ScrollIndex);
	OR_updateChecks();
	
	-- All done inviting!
	if getn(OR_inviteList) == 0 then
		OpenRaidInvList:Hide();
		print('|cffB40404[OpenRaid]: |cffB18904Have a great raid!');
		
		-- Reset list for next raid.
		OR_inviteList = {};
		OR_inviteIndex = 1;
	end
	
	-- Sometimes it doesn't invite everyone. Go again! \o/
	OR_bnetInvitesOnly();
end

-- Drain the progress bar
function OR_progressBar()
	if or_progressing == false then
		return;
	end
	
	or_progressCounter = or_progressCounter + 1;
	
	-- Drain the progress bar every 3 frames (update is called max. 60 times per second so that's good).
	if (or_progressCounter % 2 == 0) then
		if or_progressValue > 0 then
			or_progressValue = or_progressValue - 1;
		end
		OpenRaidStatusBar:SetValue(or_progressValue/100);
	end
	
	if or_progressCounter > 6000 then
		or_progressCounter = 0;
	end
end

-- Import a string from OpenRaid
function OR_import()

	-- 
	-- OpenRaid string format:
	--    <CharName>;<RaidName>,<Month>,<Day>,<Year>,<Hour>,<Minute>;...repeat raids
	--
	-- Example of the format:
	--    Cheetáh;Firelands,2,25,2012,20,0;TestRaid,2,26,2012,21,0;TestRaid 2,2,27,2012,21,0;TestRaid 3,2,28,2012,21,0;
	--
	
	local orString = {strsplit(";", OpenRaidImportEdit:GetText())};
	
	-- First is the character name
	local charName = orString[1];
	local myCharName,realm = UnitName("player");
	local added = 0;
	
	-- Is the string correct?
	if charName == '' or charName == nil or orString[2] == nil then
		print('|cffB40404[OpenRaid ERROR]: |cffB18904This OpenRaid string does not seem correct.');
	end
	
	-- This is for a different character...
	if (myCharName ~= charName) then
		print('|cffB40404[OpenRaid ERROR]: |cffB18904This OpenRaid string is for player name '..charName..'! Please log in on '..charName..' instead.');
	end
	
	local notFoundN = 0;
	
	-- Loop over all raids, start at 2 (1 = character name)
	for n=2,getn(orString) do
		
		-- Last string is empty.
		if (orString[n] == nil or orString[n] == '') then
			break;
		end
		
		-- Get this raid's deatils
		local raid = {strsplit(",", orString[n])};
		local monthOffset = abs(CalendarGetMonth(0)-raid[2]);
		local found = false;
		
		-- Is it already on the calendar? Loop over the next 2 months... this is quite fast :)
		for month=0,2 do
			for day=1,31 do
				for r=1,CalendarGetNumDayEvents(month, day) do
					local presentRaid, hours, minutes = CalendarGetDayEvent(month, day, r);
					
					if (presentRaid == "OpenRaid - "..raid[1]) then
						
						-- Do we need to update the start/end times?
						if (hours ~= tonumber(raid[5]) or minutes ~= tonumber(raid[6])) then
							CalendarOpenEvent(month, day, r);
							CalendarEventSetDate(raid[2], raid[3], raid[4]);
							CalendarEventSetTime(raid[5], raid[6]);
							CalendarUpdateEvent();
						end 
						
						-- We found a match so don't create this.
						found = true;
						break;
					end
				end
			end
		end
		
		-- Add a new calendar invite
		if (not found) then
			notFoundN = notFoundN + 1;
			if (added > 0) then
				or_progressValue = 100;
				or_progressing = true;
				OpenRaidAnnoyingForm:Show();
				OpenRaidForm:Hide();
			else
				CalendarNewEvent();
				CalendarEventSetType(1);
				CalendarEventSetDate(raid[2], raid[3], raid[4]);
				CalendarEventSetTime(raid[5], raid[6]);
				CalendarEventSetTitle("OpenRaid - "..raid[1]);
				CalendarEventSetDescription("Visit OpenRaid for more information and signups!");
				CalendarAddEvent();
				added = added + 1;
				OpenRaidForm:Hide();
			end
		end
	end
	
	if (notFoundN == 0 or (notFoundN == 1 and or_progressing == false)) then
		print('|cffB40404[OpenRaid]: |cffB18904You are all up to date, thank you for your patience!');
		OpenRaidAnnoyingForm:Hide();
		OpenRaidForm:Hide();
		or_progressing = false;
	end
end